==> Mudlib-CD.00.28.tar.gz: lib/std/player/quicktyper.c <==

/*
 *  Yet an other quicktyper version, this time built into the player object,
 *  for speed and efficiency.
 *  Original quicktyper by Tech, some code in this version by Plugh.
 *                                             /Tintin
 */

#include "/sys/macros.h"
#include "/secure/std.h"

#define CHECK_SO_WIZ 	if (WIZ_CHECK < WIZ_NORMAL) return 0; \
			if (this_interactive() != this_player()) return 0

#define	HISTORY_LENGTH	20
#define	ALIAS_LENGTH	20
#define	VERSION		"1.01"
#define	VERSION_DATE	"921013"

mixed add_history(string verb, string str);

static int history_num = 0;
static string *history_list = ({ });

static int paused = 0;
static string do_sequence = "";

static mapping	commands= ([
		    "alias"   : "alias",
		    "do"      : "doit",
		    "help"    : "help",
		    "history" : "history",
		    "resume"  : "resume",
		    "unalias" : "unalias",
		    "version" : "version"
		]);

/*
 * Function name: cmdmodify
 * Description:   This function is called from modify_command when it's time to
 *		  modify a command
 * Arguments:	  str - The command
 * Returns:	  The modified command
 */
public string
cmdmodify(string str)
{
    string  cmd, param;
    mixed   newcmd;

    if (strlen(str) < 1)
	return str;

    if (sscanf(str, "%s %s", cmd, param) <= 0)
    {
	cmd = str;
	param = 0;
    }

    newcmd = add_history(cmd, param);
    if (newcmd)
	str = newcmd;

    if (str && stringp(str) && sscanf(str, "%s %s", cmd, param) <= 0)
    {
	cmd = str;
	param = 0;
    }

    if (commands[cmd] && call_other(this_object(), commands[cmd], param))
	    return 0;
    if (strlen(str))
	return readable_string(str);
    else
	return "";
}

public void
attach(object ob)
{
    ob->set_cmdmodify(this_object());
}

/* **************************************************************************
 * Utility functions needed to make the quicktyper work
 * **************************************************************************/

/*
 * Function name: do_old
 * Description:   Try to do a command from the history list
 */
mixed
do_old(string verb, string str)
{
    string  temp, com;
    int     i, pos, len;
    
    if (!sizeof(history_list))
    {
	write("No history!\n");
	return 0;
    }

    if (verb == "%%")
    {
	com = history_list[sizeof(history_list) - 1] +
		(strlen(str) ? " " + str : "");
 	if (this_player()->query_get_echo())
	    write("Doing: " + com + "\n");
	return com;
    }

    if (sscanf(verb, "%%d%s", pos, temp) >= 1)
    {
	if (pos > history_num)
	{
	    write("History entry number " + pos + " does not yet exist.\n");
	    return 0;
	}
	else if (pos < (history_num - HISTORY_LENGTH + 1))
	{
	    write("History entry number " + pos +
		  " does not exist any longer.\n");
	    return 0;
	}
	pos--;
	if (history_num > HISTORY_LENGTH)
	    pos = pos - history_num + HISTORY_LENGTH;
	if (!temp)
	    temp = "";
	com = history_list[pos] + temp + (strlen(str) ? " " + str : "");
	if (this_player()->query_get_echo())
	    write("Doing: " + com + "\n");
 	return com;
    }

    if (sscanf(verb, "%%s", temp) == 1)
    {
	len = strlen(temp) - 1;
	for (i = sizeof(history_list) - 1; i >= 0; i--)
	{
	    if (history_list[i][0..len] == temp)
	    {
		com = history_list[i];
		if (this_player()->query_get_echo())
		    write("Doing: " + com + "\n");
		return com;
	    }
	}
	write("No previous command '" + temp + "' found!\n");
	return 1;
    }

    return 0;
}

/*
 * Function name: add_history
 * Description:   Add a command to the history list
 */
mixed
add_history(string verb, string str)
{
    string tmp;

    if (!verb || verb == "")
	return 0;

    if (strlen(verb) > 1 && verb[0] == '%')
	return do_old(verb, str);

    if (!(tmp = m_alias_list[verb]))
	tmp = verb;
	
    if (sizeof(history_list) >= HISTORY_LENGTH)
	history_list = history_list[1 .. sizeof(history_list)];
    else if (!sizeof(history_list))
	history_list = ({ });

    history_list += ({ tmp + (strlen(str) ? " " + str : "") });
    history_num++;
    return tmp + (strlen(str) ? " " + str : "");
}

/*
 * Function name: alias
 * Description:   Make an alias
 */
int
alias(string str)
{
    int     i;
    mixed   *l;
    string  a, cmd;

    if (!str || str == "")
    {
	write("You current aliases are:\n\n");
	l = m_indexes(m_alias_list);
	for (i = 0; i < sizeof(l); i++)
	    write(extract(l[i] + "           ", 0, 9) + m_alias_list[l[i]] + "\n");
	return 1;
    }
    if (sscanf(str, "%s %s", a, cmd) == 2)
    {
	if (m_alias_list[a])
	    m_alias_list = m_delete(m_alias_list, a);
	l = m_indexes(m_alias_list);
	if (sizeof(l) < ALIAS_LENGTH)
	    m_alias_list += ([ a : cmd ]);
	else
	    write("Sorry, the alias list is full!\n");
	return 1;
    }
    if (sscanf(str, "%s", a) == 1)
    {
	if (m_alias_list[a])
	    write(extract(a + "          ", 0, 9) + m_alias_list[a] + "\n");
	else
	    write("Alias '" + a + "' did not exist!\n");
	return 1;
    }
    return 0;
}

/* 
 * Function name: doit
 * Description:   Do a sequence of commands
 */
int
doit(string str)
{

    if (!str || str=="")
    {
	if (strlen(do_sequence))
	{
	    write("Paused.  Use \"resume\" to continue.\n");
	    remove_call_out("do_chain");
	    paused = 1;
	}
	else
	    write("Usage: do cmd1,cmd2,cmd3,...\n");
	return 1;
    }
    if (strlen(do_sequence) && !paused)
    {
	write("Busy executing commands:\n  " + do_sequence + "\n");
	return 1;
    }
    if (strlen(do_sequence))
        write("Skipping paused commands:\n  " + do_sequence + "\n");
    do_sequence = str;
    call_out("do_chain", 2);
    return 1;
}

/*
 * Function name: do_chain
 * Description:   Do next command in the do chain
 */
void
do_chain()
{
    string cmd, rest;

    if (sscanf(do_sequence, "%s,%s", cmd, rest) == 2)
    {
	tell_object(this_object(), "Doing: " + cmd + "\n");
	this_object()->command(cmd);
	do_sequence = rest;
	call_out("do_chain", 2);
	return;
    }
    else
    {
	tell_object(this_object(), "Doing: " + do_sequence + "\n");
	this_object()->command(do_sequence);
	tell_object(this_object(), "Done.\n");
	do_sequence = "";
    }
}

/* 
 * Function name: resume
 * Description:   Resume the processing of the commands in the do chain
 */
int
resume()
{
    if (!strlen(do_sequence) || !paused)
    {
	write("Nothing to resume.\n");
	return 1;
    }
    paused = 0;
    call_out("do_chain", 1);
    return 1;
}

/*
 * Function name: help
 * Description:   Help about the quicktyper
 */
int
help(string str)
{
    if (!str || str != "quicktyper")
	return 0;
    write(break_string("This quicktyper allows command aliases, e.g. short " +
          "commands.  These are expanded by the Quicktyper to their full " +
	  "equivalents.\n\n",76) +
"The available commands in quicktyper is:\n\n"+
"  alias                      - show the list of current aliases\n" +
"  alias command              - show the meaning of alias \"command\"\n" +
"  alias command what to do   - make \"command\" an alias for the\n" +
"                               \"what to do\"-command\n" +
"  unalias command            - remove the alias named \"command\"\n"+
"  do cmd1,cmd2,cmd3,...      - execute several commands in sequence\n"+
"  do                         - pause execution of sequenced commands\n"+
"  resume                     - resume paused commands\n"+
"  history                    - display previous "+HISTORY_LENGTH+" commands\n"+
"  %%                         - repeat last command\n"+
"  %n                         - repeat command number 'n'\n"+
"  %text                      - repeat last command starting with 'text'\n"+
"  help quicktyper            - this help text\n");
    if (this_player()->query_wiz_level() > 0)
	write("  version quicktyper         - show quicktyper version\n");
    write("\nExamples:\n\n" +
"  'alias l look at watch'    - makes 'l' a command to look at a watch\n" +
"  'do smile,look,laugh'      - makes you first smile, then look around,\n" +
"                               and at last, laugh\n\n"+
	  break_string("Giving '%%' as a command at this point, will NOT make " +
"you smile, look and laugh.  Because of the way the quicktyper is "+
"implemented, internal quicktyper commands will not be added to the " +
"history.  The same goes for aliases.\n\n",76));
    return 1;
}

/*
 * Function name: history
 * Description:   List the history of commands we have done
 */
int
history(string str)
{
    int     i, number;
    string  *h;

    for (i = 0; i < sizeof(history_list); i++)
    {
	number = history_num + i - sizeof(history_list) + 1;
	write("%" + number + "\t" + history_list[i] + "\n");
    }
    return 1;
}

/*
 * Function name: unalias
 * Description:   Remove an alias
 */
int
unalias(string str)
{
    string  a;

    if (!str || str=="")
	return 0;
    if (m_alias_list[str]) 
	m_alias_list = m_delete(m_alias_list, str);
    else
	write("Alias '" + str + "' did not exist!\n");
    return 1;
}

/*
 * Function name: version
 * Description:   Get the version of the quicktyper
 */
int
version(string str)
{
    if (!str || str!="quicktyper")
	return 0;
	
    CHECK_SO_WIZ;

    write("Quicktyper version " + VERSION +
	  " created " + VERSION_DATE + "\n");
    return 1;
}

==> Mudlib-CD.00.30.tar.gz: lib/std/player/quicktyper.c <==

/*
 *  Yet an other quicktyper version, this time built into the player object,
 *  for speed and efficiency.
 *  Original quicktyper by Tech, some code in this version by Plugh.
 *                                             /Tintin
 */

#include "/sys/macros.h"
#include "/secure/std.h"

#define CHECK_ME	if (this_interactive() != this_object()) return 0
#define CHECK_SO_WIZ 	if (WIZ_CHECK < WIZ_NORMAL) return 0; \
			if (this_interactive() != this_object()) return 0

#define	HISTORY_LENGTH	20
#define	ALIAS_LENGTH	30
#define	VERSION		"1.01"
#define	VERSION_DATE	"921013"

mixed add_history(string verb, string str);

static int history_num = 0;
static string *history_list = ({ });

static int paused = 0;
static string do_sequence = "";

static mapping	commands= ([
		    "alias"   : "alias",
		    "do"      : "doit",
		    "help"    : "help",
		    "history" : "history",
		    "resume"  : "resume",
		    "unalias" : "unalias",
		    "version" : "version"
		]);

/*
 * Function name: cmdmodify
 * Description:   This function is called from modify_command when it's time to
 *		  modify a command
 * Arguments:	  str - The command
 * Returns:	  The modified command
 */
public string
cmdmodify(string str)
{
    string  cmd, param;
    mixed   newcmd;

    if (strlen(str) < 1)
	return str;

    if (sscanf(str, "%s %s", cmd, param) <= 0)
    {
	cmd = str;
	param = 0;
    }

    newcmd = add_history(cmd, param);
    if (newcmd)
	str = newcmd;

    if (str && stringp(str) && sscanf(str, "%s %s", cmd, param) <= 0)
    {
	cmd = str;
	param = 0;
    }

    if (commands[cmd] && call_other(this_object(), commands[cmd], param))
	    return 0;
    if (strlen(str))
	return readable_string(str);
    else
	return "";
}

public void
attach(object ob)
{
    ob->set_cmdmodify(this_object());
}

/* **************************************************************************
 * Utility functions needed to make the quicktyper work
 * **************************************************************************/

/*
 * Function name: do_old
 * Description:   Try to do a command from the history list
 */
static mixed
do_old(string verb, string str)
{
    string  temp, com;
    int     i, pos, len;
    
    if (!sizeof(history_list) || !history_len)
    {
	write("No history!\n");
	return 0;
    }

    if (verb == "%%")
    {
	com = history_list[sizeof(history_list) - 1] +
		(strlen(str) ? " " + str : "");
 	if (this_player()->query_get_echo())
	    write("Doing: " + com + "\n");
	return com;
    }

    if (sscanf(verb, "%%d%s", pos, temp) >= 1)
    {
	if (pos > history_num)
	{
	    write("History entry number " + pos + " does not yet exist.\n");
	    return 0;
	}
	else if (pos < (history_num - history_len + 1))
	{
	    write("History entry number " + pos +
		  " does not exist any longer.\n");
	    return 0;
	}
	pos--;
	if (history_num > history_len)
	    pos = pos - history_num + history_len;
	if (!temp)
	    temp = "";
	com = history_list[pos] + temp + (strlen(str) ? " " + str : "");
	if (this_player()->query_get_echo())
	    write("Doing: " + com + "\n");
 	return com;
    }

    if (sscanf(verb, "%%s", temp) == 1)
    {
	len = strlen(temp) - 1;
	for (i = sizeof(history_list) - 1; i >= 0; i--)
	{
	    if (history_list[i][0..len] == temp)
	    {
		com = history_list[i];
		if (this_player()->query_get_echo())
		    write("Doing: " + com + "\n");
		return com;
	    }
	}
	write("No previous command '" + temp + "' found!\n");
	return 0;
    }

    return 0;
}

/*
 * Function name: add_history
 * Description:   Add a command to the history list
 */
static mixed
add_history(string verb, string str)
{
    string tmp;

    if (!verb || verb == "")
	return 0;

    if (strlen(verb) > 1 && verb[0] == '%')
	return do_old(verb, str);

    if (!(tmp = m_alias_list[verb]))
	tmp = verb;

    if (history_len)
    {
	if (sizeof(history_list) >= history_len)
	    history_list = history_list[1 .. sizeof(history_list)];
	else if (!sizeof(history_list))
	    history_list = ({ });
	history_list += ({ tmp + (strlen(str) ? " " + str : "") });
	history_num++;
    }
	
    return tmp + (strlen(str) ? " " + str : "");
}

/*
 * Function name: alias
 * Description:   Make an alias
 */
int
alias(string str)
{
    int     i;
    mixed   *l;
    string  a, cmd;

    CHECK_ME;
    
    if (!str || str == "")
    {
	write("You current aliases are:\n\n");
	l = m_indexes(m_alias_list);
	for (i = 0; i < sizeof(l); i++)
	    write(extract(l[i] + "           ", 0, 9) + m_alias_list[l[i]] + "\n");
	return 1;
    }
    if (sscanf(str, "%s %s", a, cmd) == 2)
    {
	if (m_alias_list[a])
	    m_alias_list = m_delete(m_alias_list, a);
	l = m_indexes(m_alias_list);
	if (sizeof(l) < ALIAS_LENGTH)
	    m_alias_list += ([ a : cmd ]);
	else
	    write("Sorry, the alias list is full!\n");
	return 1;
    }
    if (sscanf(str, "%s", a) == 1)
    {
	if (m_alias_list[a])
	    write(extract(a + "          ", 0, 9) + m_alias_list[a] + "\n");
	else
	    write("Alias '" + a + "' did not exist!\n");
	return 1;
    }
    return 0;
}

/* 
 * Function name: doit
 * Description:   Do a sequence of commands
 */
int
doit(string str)
{
    CHECK_ME;
    
    if (!str || str=="")
    {
	if (strlen(do_sequence))
	{
	    write("Paused.  Use \"resume\" to continue.\n");
	    remove_call_out("do_chain");
	    paused = 1;
	}
	else
	    write("Usage: do cmd1,cmd2,cmd3,...\n");
	return 1;
    }
    if (strlen(do_sequence) && !paused)
    {
	write("Busy executing commands:\n  " + do_sequence + "\n");
	return 1;
    }
    if (strlen(do_sequence))
        write("Skipping paused commands:\n  " + do_sequence + "\n");
    do_sequence = str;
    call_out("do_chain", 2);
    return 1;
}

/*
 * Function name: do_chain
 * Description:   Do next command in the do chain
 */
void
do_chain()
{
    string cmd, rest;

    if (sscanf(do_sequence, "%s,%s", cmd, rest) == 2)
    {
	tell_object(this_object(), "Doing: " + cmd + "\n");
	this_object()->command(cmd);
	do_sequence = rest;
	call_out("do_chain", 2);
	return;
    }
    else
    {
	tell_object(this_object(), "Doing: " + do_sequence + "\n");
	this_object()->command(do_sequence);
	tell_object(this_object(), "Done.\n");
	do_sequence = "";
    }
}

/* 
 * Function name: resume
 * Description:   Resume the processing of the commands in the do chain
 */
int
resume()
{
    CHECK_ME;
    
    if (!strlen(do_sequence) || !paused)
    {
	write("Nothing to resume.\n");
	return 1;
    }
    paused = 0;
    call_out("do_chain", 1);
    return 1;
}

/*
 * Function name: help
 * Description:   Help about the quicktyper
 */
int
help(string str)
{
    CHECK_ME;
    
    if (!str || str != "quicktyper")
	return 0;
    write(break_string("This quicktyper allows command aliases, e.g. short " +
          "commands.  These are expanded by the Quicktyper to their full " +
	  "equivalents.\n\n",76) +
"The available commands in quicktyper is:\n\n"+
"  alias                      - show the list of current aliases\n" +
"  alias command              - show the meaning of alias \"command\"\n" +
"  alias command what to do   - make \"command\" an alias for the\n" +
"                               \"what to do\"-command\n" +
"  unalias command            - remove the alias named \"command\"\n"+
"  do cmd1,cmd2,cmd3,...      - execute several commands in sequence\n"+
"  do                         - pause execution of sequenced commands\n"+
"  resume                     - resume paused commands\n"+
"  history                    - display previous "+history_len+" commands\n"+
"                               with a numerical argument, it sets the\n" + 
"                               length of the history list.\n" +
"  %%                         - repeat last command\n"+
"  %n                         - repeat command number 'n'\n"+
"  %text                      - repeat last command starting with 'text'\n"+
"  help quicktyper            - this help text\n");
    if (this_player()->query_wiz_level() > 0)
	write("  version quicktyper         - show quicktyper version\n");
    write("\nExamples:\n\n" +
"  'alias l look at watch'    - makes 'l' a command to look at a watch\n" +
"  'do smile,look,laugh'      - makes you first smile, then look around,\n" +
"                               and at last, laugh\n\n"+
	  break_string("Giving '%%' as a command at this point, will NOT make " +
"you smile, look and laugh.  Because of the way the quicktyper is "+
"implemented, internal quicktyper commands will not be added to the " +
"history.  The same goes for aliases.\n\n",76));
    return 1;
}

/*
 * Function name: history
 * Description:   List the history of commands we have done
 */
int
history(string str)
{
    int     i, number;
    string  *h;

    CHECK_ME;

    if (strlen(str))
    {
	number = atoi(str);

	if (number < 0 || number > HISTORY_LENGTH)
	{
	    notify_fail("0 >= history length < 20.\n");
	    return 0;
	}

	history_len = number;
	if (number == 0)
	    history_list = ({ });
	write("Ok.\n");
	return 1;
    }

    if (history_len)
    {
	for (i = 0; i < sizeof(history_list); i++)
	{
	    number = history_num + i - sizeof(history_list) + 1;
	    write("%" + number + "\t" + history_list[i] + "\n");
	}
    }
    return 1;
}

/*
 * Function name: unalias
 * Description:   Remove an alias
 */
int
unalias(string str)
{
    string  a;

    CHECK_ME;
    
    if (!str || str=="")
	return 0;
    if (m_alias_list[str]) 
	m_alias_list = m_delete(m_alias_list, str);
    else
	write("Alias '" + str + "' did not exist!\n");
    return 1;
}

/*
 * Function name: version
 * Description:   Get the version of the quicktyper
 */
int
version(string str)
{
    if (!str || str!="quicktyper")
	return 0;
	
    CHECK_SO_WIZ;

    write("Quicktyper version " + VERSION +
	  " created " + VERSION_DATE + "\n");
    return 1;
}
